"""
Async DRACOON users adapter based on httpx and pydantic
V1.2.0
(c) Octavio Simone, November 2021 

Collection of DRACOON API calls for user management
Documentation: https://dracoon.team/api/swagger-ui/index.html?configUrl=/api/spec_v4/swagger-config#/user

Please note: maximum 500 items are returned in GET requests 

 - refer to documentation for details on filtering and offset 
 - use documentation for payload description 
 
All requests with bodies use generic params variable to pass JSON body

"""

import httpx
import logging
from pydantic import validate_arguments

from dracoon.client import DRACOONClient, OAuth2ConnectionType
from dracoon.crypto import create_plain_userkeypair, encrypt_private_key
from dracoon.crypto.models import UserKeyPairContainer, UserKeyPairVersion
from dracoon.errors import ClientDisconnectedError, InvalidClientError, InvalidArgumentError
from .models import UpdateAccount, UserAccount




class DRACOONUser:
    """
    API wrapper for DRACOON user endpoint:
    Account information and management (user profile)
    """

    def __init__(self, dracoon_client: DRACOONClient):
        """ requires a DRACOONClient to perform any request """
        if not isinstance(dracoon_client, DRACOONClient):
            raise InvalidClientError(message='Invalid DRACOON client format.')
        if dracoon_client.connection:
            self.dracoon = dracoon_client
            self.api_url = self.dracoon.base_url + self.dracoon.api_base_url + '/user'
            self.logger = logging.getLogger('dracoon.user')
            if self.dracoon.raise_on_err:
                self.raise_on_err = True
            else:
                self.raise_on_err = False
            self.logger.debug("DRACOON user adapter created.")
        else:
            self.logger.error("DRACOON client error: no connection. ")
            raise ClientDisconnectedError(message='DRACOON client must be connected: client.connect()')


    # get account information for current user
    @validate_arguments
    async def get_account_information(self, more_info: bool = False, raise_on_err: bool = False) -> UserAccount:
        """ returns account information for authenticated user """
        if not await self.dracoon.test_connection() and self.dracoon.connection:
            await self.dracoon.connect(OAuth2ConnectionType.refresh_token)

        if self.raise_on_err:
            raise_on_err = True

        try:
            api_url = self.api_url + f'/account/?{str(more_info).lower()}'
            res = await self.dracoon.http.get(api_url)
            res.raise_for_status()
        except httpx.RequestError as e:
            await self.dracoon.handle_connection_error(e)
        except httpx.HTTPStatusError as e:
            self.logger.error("Getting account information failed.")
            await self.dracoon.handle_http_error(err=e, raise_on_err=raise_on_err)
        
        self.logger.info("Retrieved user account.")
        return UserAccount(**res.json())

    # update account information for current user
    @validate_arguments
    async def update_account_information(self, account_update: UpdateAccount, raise_on_err: bool = False) -> UserAccount:
        """ updates account information for authenticated user """
        if not await self.dracoon.test_connection() and self.dracoon.connection:
            await self.dracoon.connect(OAuth2ConnectionType.refresh_token)

        if self.raise_on_err:
            raise_on_err = True

        api_url = self.api_url + f'/account'

        payload = account_update.dict(exclude_unset=True)

        try:
            res = await self.dracoon.http.put(url=api_url, json=payload)
            res.raise_for_status()
        except httpx.RequestError as e:
            await self.dracoon.handle_connection_error(e)
        except httpx.HTTPStatusError as e:
            self.logger.error("Updating account information failed.")
            await self.dracoon.handle_http_error(err=e, raise_on_err=raise_on_err)
        
        self.logger.info("Updated user account.")
        return UserAccount(**res.json())

    def make_account_update(self, user_name: str = None, acceptEULA: bool = None, first_name: str = None, last_name: str = None, email: str = None, 
                            phone: str = None, language: str = None) -> UpdateAccount:
        """ creates account information update payload – required for update_account_information() """
        account_update = {}

        if acceptEULA == False:
            raise InvalidArgumentError(message="EULA acceptance cannot be undone.")

        if language: account_update["language"] = language
        if phone: account_update["phone"] = phone
        if first_name: account_update["firstName"] = first_name  
        if last_name: account_update["lastName"] = last_name    
        if email: account_update["email"] = email 
        if acceptEULA is not None: account_update["acceptEULA"] = acceptEULA
        if user_name: account_update["userName"] = user_name

        return UpdateAccount(**account_update)

    # get user keypair (encrypted)
    @validate_arguments
    async def get_user_keypair(self, version: UserKeyPairVersion = None, raise_on_err: bool = False) -> UserKeyPairContainer:
        """ returns encrypted user keypair (if present) for authenticated user """
        if not await self.dracoon.test_connection() and self.dracoon.connection:
            await self.dracoon.connect(OAuth2ConnectionType.refresh_token)

        if self.raise_on_err:
            raise_on_err = True

        api_url = self.api_url + f'/account/keypair'
        if version != None: api_url += '/?version=' + version.value

        try:
            res = await self.dracoon.http.get(api_url)
            res.raise_for_status()
        except httpx.RequestError as e:
            await self.dracoon.handle_connection_error(e)
        except httpx.HTTPStatusError as e:
            self.logger.error("Getting user keypair failed.")
            await self.dracoon.handle_http_error(err=e, raise_on_err=raise_on_err)
        
        self.logger.info("Retrieved user keypair.")
        return UserKeyPairContainer(**res.json())

    # set user keypair 
    @validate_arguments
    async def set_user_keypair(self, secret: str, version: UserKeyPairVersion = UserKeyPairVersion.RSA4096, raise_on_err: bool = False) -> None:
        """ sets encrypted user keypair protected with secret (if none present) for authenticated user """
        plain_keypair = create_plain_userkeypair(version=version)
        encrypted_keypair = encrypt_private_key(secret=secret, plain_key=plain_keypair)

        if self.raise_on_err:
            raise_on_err = True
        
        payload = encrypted_keypair.dict(exclude_unset=True)

        if not await self.dracoon.test_connection() and self.dracoon.connection:
            await self.dracoon.connect(OAuth2ConnectionType.refresh_token)

        api_url = self.api_url + f'/account/keypair'

        try:
            res = await self.dracoon.http.post(url=api_url, json=payload)

            res.raise_for_status()
        except httpx.RequestError as e:
            await self.dracoon.handle_connection_error(e)
        except httpx.HTTPStatusError as e:
            self.logger.error("Setting user keypair failed.")
            await self.dracoon.handle_http_error(err=e, raise_on_err=raise_on_err)
        
        self.logger.info("Set user keypair.")
        return None

    # delete user keypair 
    @validate_arguments
    async def delete_user_keypair(self, version: UserKeyPairVersion = None, raise_on_err: bool = False) -> None:
        """ deletes encrypted user keypair (if present) for authenticated user """
        if not await self.dracoon.test_connection() and self.dracoon.connection:
            await self.dracoon.connect(OAuth2ConnectionType.refresh_token)

        if self.raise_on_err:
            raise_on_err = True

        api_url = self.api_url + f'/account/keypair'
        if version != None: api_url += '/?version=' + version.value

        try:
            res = await self.dracoon.http.delete(api_url)
            res.raise_for_status()
        except httpx.RequestError as e:
            await self.dracoon.handle_connection_error(e)
        except httpx.HTTPStatusError as e:
            self.logger.error("Deleting user keypair failed.")
            await self.dracoon.handle_http_error(err=e, raise_on_err=raise_on_err)
        
        self.logger.info("Deleted user keypair.")
        return None



from pydantic import BaseModel
from typing import Optional, List
from datetime import datetime

from dracoon.nodes.models import NodeType, UserInfo
from dracoon.client.models import Range

class DownloadShare(BaseModel):
    id: int
    name: str
    nodeId: int
    accessKey: str
    cntDownloads: int
    createdAt: datetime
    createdBy: UserInfo
    updatedAt: Optional[datetime]
    updatedBy: Optional[UserInfo]
    notes: Optional[str]
    internalNotes: Optional[str]
    showCreatorName: Optional[bool]
    showCreatorUsername: Optional[bool]
    isProtected: Optional[bool]
    expireAt: Optional[datetime]
    maxDownloads: Optional[int]
    nodePath: Optional[str]
    dataUrl: Optional[str]
    isEncrypted: Optional[bool]
    nodeType: Optional[NodeType]

class DownloadShareList(BaseModel):
    range: Range
    items: List[DownloadShare]

class UploadShare(BaseModel):
    id: int
    name: str
    targetId: int
    isProtected: bool
    accessKey: str
    createdAt: datetime
    createdBy: UserInfo
    updatedAt: Optional[datetime]
    updatedBy: Optional[UserInfo]
    expireAt: Optional[datetime]
    targetPath: Optional[str]
    isEncrypted: Optional[bool]
    notes: Optional[str]
    internalNotes: Optional[str]
    filesExpiryPeriod: Optional[int]
    cntFiles: Optional[int]
    cntUploads: Optional[int]
    showUploadedFiles: Optional[bool]
    dataUrl: Optional[str]
    maxSlots: Optional[int]
    maxSite: Optional[int]
    targetType: Optional[NodeType]
    showCreatorName: Optional[bool]
    showCreatorUsername: Optional[bool]

class UploadShareList(BaseModel):
    range: Range
    items: List[UploadShare]





from dracoon.nodes.models import Node
from dracoon.nodes.responses import S3FileUploadStatus
from dracoon import DRACOON, OAuth2ConnectionType
import dotenv
import os
import shutil
import asyncio
from pathlib import Path



dotenv.load_dotenv()

client_id = os.environ.get('CLIENT_ID')
client_secret = os.environ.get('CLIENT_SECRET')
username = os.environ.get('E2E_USER_NAME')
password = os.environ.get('E2E_PASSWORD')
base_url = os.environ.get('E2E_BASE_URL')
base_url_server = os.environ.get('E2E_SERVER_BASE_URL')
upload_target = os.environ.get('E2E_UPLOAD_TARGET')
upload_crypto_target = os.environ.get('E2E_CRYPTO_UPLOAD_TARGET')
upload_file_small = os.environ.get('E2E_UPLOAD_SOURCE_SMALL')
upload_file_large = os.environ.get('E2E_UPLOAD_SOURCE_LARGE')

download_target = os.environ.get('E2E_DOWNLOAD_TARGET')

async def test_transfers_e2e():
    dracoon = DRACOON(base_url=base_url, client_id=client_id, client_secret=client_secret)
    await dracoon.connect(OAuth2ConnectionType.password_flow, username=username, password=password)
    
    await dracoon.user.set_user_keypair('VerySecret123!')
    
    await dracoon.get_keypair('VerySecret123!')
    
    file_small = await dracoon.upload(target_path=upload_target, file_path=upload_file_small)
    assert isinstance(file_small, S3FileUploadStatus)
    print('Uploading small file (no chunking) test passed (/)')
    
    
    file_large = await dracoon.upload(target_path=upload_target, file_path=upload_file_large, display_progress=True)
    assert isinstance(file_large, S3FileUploadStatus)
    print('Uploading large file (chunking) test passed (/)')
    

    crypto_file_small = await dracoon.upload(target_path=upload_crypto_target, file_path=upload_file_small)
    assert isinstance(crypto_file_small, S3FileUploadStatus)
    print('Uploading small file (no chunking, encrypted) test passed (/)')
    
    
    crypto_file_large = await dracoon.upload(target_path=upload_crypto_target, file_path=upload_file_large, display_progress=True)
    assert isinstance(crypto_file_large, S3FileUploadStatus)
    print('Uploading large file (chunking, encrypted) test passed (/)')
    
    tmp_dir = os.path.join(download_target, 'e2etest')
    crypto_tmp_dir = os.path.join(download_target, 'e2etest_crypto')
    
    os.mkdir(path=tmp_dir)
    os.mkdir(path=crypto_tmp_dir)
    
    download_path_small = file_small.node.parentPath + f'{file_small.node.name}'
    download_path_large = file_large.node.parentPath + f'{file_large.node.name}'
    
    crypto_download_path_small = crypto_file_small.node.parentPath + f'{crypto_file_small.node.name}'
    crypto_download_path_large = crypto_file_large.node.parentPath + f'{crypto_file_large.node.name}'
    
    await dracoon.download(file_path=download_path_small, target_path=tmp_dir)
    
    small_file = Path(os.path.join(tmp_dir, file_small.node.name))
    assert small_file.exists() and small_file.is_file() 
    assert os.stat(os.path.join(tmp_dir, file_small.node.name)).st_size == file_small.node.size
    
    print('Downloading small file (no chunking) test passed (/)')
    
    await dracoon.download(file_path=download_path_small, target_path=tmp_dir, file_name='test.test')
    
    small_file = Path(os.path.join(tmp_dir, f'test.test'))
    assert small_file.exists() and small_file.is_file() 
    assert small_file.name == 'test.test'
    assert os.stat(os.path.join(tmp_dir, file_small.node.name)).st_size == file_small.node.size
    
    print('Downloading small file (no chunking) with custom file name test passed (/)')
    
    await dracoon.download(file_path=download_path_large, target_path=tmp_dir, display_progress=True)
    
    large_file = Path(os.path.join(tmp_dir, file_large.node.name))
    assert large_file.exists() and large_file.is_file() 
    assert os.stat(os.path.join(tmp_dir, file_large.node.name)).st_size == file_large.node.size
    
    print('Downloading large file (no chunking, stream) test passed (/)')
    
    await dracoon.download(file_path=crypto_download_path_small, target_path=crypto_tmp_dir)
    
    small_crypto_file = Path(os.path.join(crypto_tmp_dir, crypto_file_small.node.name))
    assert small_crypto_file.exists() and small_crypto_file.is_file() 
    assert os.stat(os.path.join(crypto_tmp_dir, crypto_file_small.node.name)).st_size == crypto_file_small.node.size
    
    print('Downloading small file (no chunking, encrypted) test passed (/)')
    
    await dracoon.download(file_path=crypto_download_path_large, target_path=crypto_tmp_dir, display_progress=True)
    
    large_crypto_file = Path(os.path.join(crypto_tmp_dir, crypto_file_large.node.name))
    assert large_crypto_file.exists() and large_crypto_file.is_file() 
    assert os.stat(os.path.join(crypto_tmp_dir, crypto_file_large.node.name)).st_size == crypto_file_large.node.size
    
    print('Downloading large file (chunking, encrypted) test passed (/)')
    
    try:
        shutil.rmtree(tmp_dir)
        shutil.rmtree(crypto_tmp_dir)
    except OSError as e:
        print("Could not delete test folder.")
        
    await dracoon.user.delete_user_keypair()
    await dracoon.logout()
        
    print('Down- and upload (transfers) tests passed (/)')

async def test_transfers_e2e_server():
    dracoon = DRACOON(base_url=base_url_server, client_id=client_id, client_secret=client_secret)
    await dracoon.connect(OAuth2ConnectionType.password_flow, username=username, password=password)
    
    await dracoon.user.set_user_keypair('VerySecret123!')
    
    await dracoon.get_keypair('VerySecret123!')
    
    file_small = await dracoon.upload(target_path=upload_target, file_path=upload_file_small)
    assert isinstance(file_small, Node)
    print('Uploading small file (no chunking) test passed (/)')
    
    
    file_large = await dracoon.upload(target_path=upload_target, file_path=upload_file_large, display_progress=True)
    assert isinstance(file_large, Node)
    print('Uploading large file (chunking) test passed (/)')
    

    crypto_file_small = await dracoon.upload(target_path=upload_crypto_target, file_path=upload_file_small)
    assert isinstance(crypto_file_small, Node)
    print('Uploading small file (no chunking, encrypted) test passed (/)')
    
    
    crypto_file_large = await dracoon.upload(target_path=upload_crypto_target, file_path=upload_file_large, display_progress=True)
    assert isinstance(crypto_file_large, Node)
    print('Uploading large file (chunking, encrypted) test passed (/)')
    
    tmp_dir = os.path.join(download_target, 'e2etest_server')
    crypto_tmp_dir = os.path.join(download_target, 'e2etest_crypto_server')
    
    os.mkdir(path=tmp_dir)
    os.mkdir(path=crypto_tmp_dir)
    
    download_path_small = file_small.parentPath + f'{file_small.name}'
    download_path_large = file_large.parentPath + f'{file_large.name}'
    
    crypto_download_path_small = crypto_file_small.parentPath + f'{crypto_file_small.name}'
    crypto_download_path_large = crypto_file_large.parentPath + f'{crypto_file_large.name}'
    
    await dracoon.download(file_path=download_path_small, target_path=tmp_dir)
    
    small_file = Path(os.path.join(tmp_dir, file_small.name))
    assert small_file.exists() and small_file.is_file() 
    assert os.stat(os.path.join(tmp_dir, file_small.name)).st_size == file_small.size
    
    print('Downloading small file (no chunking) test passed (/)')
    
    await dracoon.download(file_path=download_path_large, target_path=tmp_dir, display_progress=True)
    
    large_file = Path(os.path.join(tmp_dir, file_large.name))
    assert large_file.exists() and large_file.is_file() 
    assert os.stat(os.path.join(tmp_dir, file_large.name)).st_size == file_large.size
    
    print('Downloading large file (no chunking, stream) test passed (/)')
    
    await dracoon.download(file_path=crypto_download_path_small, target_path=crypto_tmp_dir)
    
    small_crypto_file = Path(os.path.join(crypto_tmp_dir, crypto_file_small.name))
    assert small_crypto_file.exists() and small_crypto_file.is_file() 
    assert os.stat(os.path.join(crypto_tmp_dir, crypto_file_small.name)).st_size == crypto_file_small.size
    
    print('Downloading small file (no chunking, encrypted) test passed (/)')
    
    await dracoon.download(file_path=crypto_download_path_large, target_path=crypto_tmp_dir, display_progress=True)
    
    large_crypto_file = Path(os.path.join(crypto_tmp_dir, crypto_file_large.name))
    assert large_crypto_file.exists() and large_crypto_file.is_file() 
    assert os.stat(os.path.join(crypto_tmp_dir, crypto_file_large.name)).st_size == crypto_file_large.size
    
    print('Downloading large file (chunking, encrypted) test passed (/)')
    
    try:
        shutil.rmtree(tmp_dir)
        shutil.rmtree(crypto_tmp_dir)
    except OSError as e:
        print("Could not delete test folder.")
        
    await dracoon.user.delete_user_keypair()
    await dracoon.logout()
        
    print('Down- and upload (transfers) tests passed (/)')

        
if __name__ == '__main__':
    asyncio.run(test_transfers_e2e())
    asyncio.run(test_transfers_e2e_server())


from dracoon.client import DRACOONClient, OAuth2ConnectionType
from dracoon.user.responses import AttributesResponse, LastAdminUserRoomList, UserData, UserGroupList, UserList, RoleList
from dracoon.users import DRACOONUsers
import dotenv
import os
import asyncio
from dracoon.users.models import AttributeEntry, CreateUser, UpdateUser, UpdateUserAttributes, UserAuthData


dotenv.load_dotenv()

client_id = os.environ.get('CLIENT_ID')
client_secret = os.environ.get('CLIENT_SECRET')
username = os.environ.get('E2E_USER_NAME')
password = os.environ.get('E2E_PASSWORD')
base_url = os.environ.get('E2E_BASE_URL')
base_url_server = os.environ.get('E2E_SERVER_BASE_URL')


async def test_users_e2e():

    dracoon = DRACOONClient(base_url=base_url, client_id=client_id, client_secret=client_secret)
    await dracoon.connect(OAuth2ConnectionType.password_flow, username=username, password=password)
    users = DRACOONUsers(dracoon_client=dracoon)
    assert isinstance(users, DRACOONUsers)
    print('Connection test complete (/)')

    local_user = users.make_local_user(first_name='test', last_name='test', email='test@unbekanntespferd.com')
    assert isinstance(local_user, CreateUser)
    print('Local user payload creation test passed (/)')

    ad_user = users.make_ad_user(first_name='test', last_name='test', email='test@unbekanntespferd.com', login='ad.login', ad_id=4)
    assert isinstance(ad_user, CreateUser)
    print('AD user payload creation test passed (/)')
    
    oidc_user = users.make_oidc_user(first_name='test', last_name='test', email='test@unbekanntespferd.com', login='oidc.login', oidc_id=5)
    assert isinstance(oidc_user, CreateUser)
    print('OIDC user creation test passed (/)')

    update = users.make_user_update(first_name='test')
    assert isinstance(update, UpdateUser)
    print('User update payload creation test passed (/)')

    auth = users.make_auth_data(method='openid')
    assert isinstance(auth, UserAuthData)
    print('User auth data payload creation test passed (/)')
    

    user_list = await users.get_users()
    assert isinstance(user_list, UserList)
    print('Getting users test passed (/)')
    print(f'Fetched {user_list.range.total} users')

    local_user = users.make_local_user(first_name='test', last_name='test', email='test@unbekanntespferd.com')

    user = await users.create_user(local_user)
    assert isinstance(user, UserData)
    print('Local user creation test passed (/)')

    res = await users.delete_user(user.id)

    assert res == None
    print('User deletion passed (/)')

    oidc_user_res = await users.create_user(oidc_user)
    ad_user_res = await users.create_user(ad_user)

    assert isinstance(oidc_user_res, UserData)
    print('OIDC user creation passed (/)')
    assert isinstance(ad_user_res, UserData)
    print('AD user creation passed (/)')

    user_details = await users.get_user(user_id=oidc_user_res.id)
    assert isinstance(user_details, UserData)
    print('Getting user passed (/)')

    user_update = users.make_user_update(first_name='Test123')

    updated_user = await users.update_user(user_id=ad_user_res.id, user_update=user_update)

    assert isinstance(updated_user, UserData)
    assert updated_user.firstName == 'Test123'
    print('User update passed (/)')

    user_groups = await users.get_user_groups(user_id=oidc_user_res.id)

    assert isinstance(user_groups, UserGroupList)
    assert len(user_groups.items) == 0

    print('Getting user groups passed (/)')

    user_last_admin_rooms = await users.get_user_last_admin_rooms(user_id=ad_user_res.id)
    assert isinstance(user_last_admin_rooms, LastAdminUserRoomList)
    assert len(user_last_admin_rooms.items) == 0

    print('Getting last admin rooms passed (/)')

    user_roles = await users.get_user_roles(user_id=oidc_user_res.id)
    assert isinstance(user_roles, RoleList)

    print('Getting user roles passed (/)')

    user_attributes = await users.get_user_attributes(user_id=oidc_user_res.id)
    assert isinstance(user_attributes, AttributesResponse)

    print('Getting user attributes passed (/)')

    user_attribute_payload = users.make_custom_user_attribute(key='test', value='test')
    user_attribute_payload2 = users.make_custom_user_attribute(key='test2', value='test2')
    attribute_list = [user_attribute_payload, user_attribute_payload2]

    assert isinstance(user_attribute_payload, AttributeEntry)

    print('Creating user attribute payload passed (/)')

    attributes_update = users.make_attributes_update(list=attribute_list)

    assert isinstance(attributes_update, UpdateUserAttributes)

    print('Creating attributes update payload passed (/)')

    attributes_update_res = await users.update_user_attributes(user_id=oidc_user_res.id, attributes=attributes_update)
    assert isinstance(attributes_update_res, UserData)

    print('Updating user attributes passed (/)')

    attr_del = await users.delete_user_attribute(user_id=oidc_user_res.id, key='test')

    assert attr_del == None

    await users.delete_user_attribute(user_id=oidc_user_res.id, key='test2')

    await users.delete_user(user_id=oidc_user_res.id)
    await users.delete_user(user_id=ad_user_res.id)

    print('Users E2E tests passed (/)')

async def test_users_e2e_server():

    dracoon = DRACOONClient(base_url=base_url_server, client_id=client_id, client_secret=client_secret)
    await dracoon.connect(OAuth2ConnectionType.password_flow, username=username, password=password)
    users = DRACOONUsers(dracoon_client=dracoon)
    assert isinstance(users, DRACOONUsers)
    print('Connection test complete (/)')

    local_user = users.make_local_user(first_name='test', last_name='test', email='test@unbekanntespferd.com')
    assert isinstance(local_user, CreateUser)
    print('Local user payload creation test passed (/)')

    ad_user = users.make_ad_user(first_name='test', last_name='test', email='test@unbekanntespferd.com', login='ad.login', ad_id=4)
    assert isinstance(ad_user, CreateUser)
    print('AD user payload creation test passed (/)')
    
    oidc_user = users.make_oidc_user(first_name='test', last_name='test', email='test@unbekanntespferd.com', login='oidc.login', oidc_id=5)
    assert isinstance(oidc_user, CreateUser)
    print('OIDC user creation test passed (/)')

    update = users.make_user_update(first_name='test')
    assert isinstance(update, UpdateUser)
    print('User update payload creation test passed (/)')

    auth = users.make_auth_data(method='openid')
    assert isinstance(auth, UserAuthData)
    print('User auth data payload creation test passed (/)')
    
    user_list = await users.get_users()
    assert isinstance(user_list, UserList)
    print('Getting users test passed (/)')
    print(f'Fetched {user_list.range.total} users')

    local_user = users.make_local_user(first_name='test', last_name='test', email='test@unbekanntespferd.com')

    user = await users.create_user(local_user)
    assert isinstance(user, UserData)
    print('Local user creation test passed (/)')

    # oidc_user_res = await users.create_user(oidc_user)
    # ad_user_res = await users.create_user(ad_user)

    # assert isinstance(oidc_user_res, UserData)
    # print('OIDC user creation passed (/)')
    # assert isinstance(ad_user_res, UserData)
    # print('AD user creation passed (/)')

    user_details = await users.get_user(user_id=user.id)
    assert isinstance(user_details, UserData)
    print('Getting user passed (/)')

    user_update = users.make_user_update(first_name='Test123')

    updated_user = await users.update_user(user_id=user.id, user_update=user_update)

    assert isinstance(updated_user, UserData)
    assert updated_user.firstName == 'Test123'
    print('User update passed (/)')

    user_groups = await users.get_user_groups(user_id=user.id)

    assert isinstance(user_groups, UserGroupList)
    assert len(user_groups.items) == 0

    print('Getting user groups passed (/)')

    user_last_admin_rooms = await users.get_user_last_admin_rooms(user_id=user.id)
    assert isinstance(user_last_admin_rooms, LastAdminUserRoomList)
    assert len(user_last_admin_rooms.items) == 0

    print('Getting last admin rooms passed (/)')

    user_roles = await users.get_user_roles(user_id=user.id)
    assert isinstance(user_roles, RoleList)

    print('Getting user roles passed (/)')

    user_attributes = await users.get_user_attributes(user_id=user.id)
    assert isinstance(user_attributes, AttributesResponse)

    print('Getting user attributes passed (/)')

    user_attribute_payload = users.make_custom_user_attribute(key='test', value='test')
    user_attribute_payload2 = users.make_custom_user_attribute(key='test2', value='test2')
    attribute_list = [user_attribute_payload, user_attribute_payload2]

    assert isinstance(user_attribute_payload, AttributeEntry)

    print('Creating user attribute payload passed (/)')

    attributes_update = users.make_attributes_update(list=attribute_list)

    assert isinstance(attributes_update, UpdateUserAttributes)

    print('Creating attributes update payload passed (/)')

    attributes_update_res = await users.update_user_attributes(user_id=user.id, attributes=attributes_update)
    assert isinstance(attributes_update_res, UserData)

    print('Updating user attributes passed (/)')

    attr_del = await users.delete_user_attribute(user_id=user.id, key='test')

    assert attr_del == None

    await users.delete_user_attribute(user_id=user.id, key='test2')

    #await users.delete_user(user_id=oidc_user_res.id)
    #await users.delete_user(user_id=ad_user_res.id)
    res = await users.delete_user(user.id)

    assert res == None
    print('User deletion passed (/)')

    print('Users E2E tests passed (/)')


if __name__ == '__main__':
    asyncio.run(test_users_e2e())
    asyncio.run(test_users_e2e_server()) 

    

   





from typing import List, Optional
from pydantic import BaseModel
from datetime import datetime

from dracoon.crypto.models import PublicKeyContainer
from dracoon.users.models import UserAuthData
from dracoon.client.models import Range


class UserGroup(BaseModel):
    id: int
    isMember: bool
    name: str

class Right(BaseModel):
    id: int
    name: str
    description: str

class Role(BaseModel):
    id: int
    name: str
    description: str
    items: Optional[List[Right]]

class RoleList(BaseModel):
    items: List[Role]

class UserItem(BaseModel):
    id: int
    userName: str
    firstName: str
    lastName: str
    isLocked: bool
    avatarUuid: str
    createdAt: Optional[datetime]
    lastLoginSuccessAt: Optional[datetime]
    expireAt: Optional[datetime]
    isEncryptionEnabled: Optional[bool]
    email: Optional[str]
    phone: Optional[str]
    homeRoomId: Optional[int]
    userRoles: Optional[RoleList]

class UserList(BaseModel):
    range: Range
    items: List[UserItem]


class UserData(BaseModel):
    id: int
    userName: str
    firstName: str
    lastName: str
    isLocked: bool
    avatarUuid: str
    authData: UserAuthData
    email: Optional[str]
    phone: Optional[str]
    expireAt: Optional[datetime]
    hasManageableRooms: Optional[bool]
    isEncryptionEnabled: Optional[bool]
    lastLoginSuccessAt: Optional[datetime]
    homeRoomId: Optional[int]
    publicKeyContainer: Optional[PublicKeyContainer]
    userRoles: Optional[RoleList]


class UserGroupList(BaseModel):
    range: Range
    items: List[UserGroup]


class LastAdminUserRoom(BaseModel):
    id: int
    name: str
    parentPath: str
    lastAdminInGroup: bool
    parentId: Optional[int]
    lastAdminInGroupId: Optional[int]


class LastAdminUserRoomList(BaseModel):
    items: List[LastAdminUserRoom]


class KeyValueEntry(BaseModel):
    key: str
    value: str

class AttributesResponse(BaseModel):
    range: Range
    items: List[KeyValueEntry]







